from libghdl import libghdl

Enable_Warning = libghdl.errorout__enable_warning



class Msgid:
    Msgid_Note = 0
    Warnid_Library = 1
    Warnid_Missing_Xref = 2
    Warnid_Default_Binding = 3
    Warnid_Binding = 4
    Warnid_Port = 5
    Warnid_Reserved_Word = 6
    Warnid_Pragma = 7
    Warnid_Nested_Comment = 8
    Warnid_Directive = 9
    Warnid_Parenthesis = 10
    Warnid_Vital_Generic = 11
    Warnid_Delayed_Checks = 12
    Warnid_Body = 13
    Warnid_Specs = 14
    Warnid_Universal = 15
    Warnid_Port_Bounds = 16
    Warnid_Runtime_Error = 17
    Warnid_Delta_Cycle = 18
    Warnid_Shared = 19
    Warnid_Hide = 20
    Warnid_Unused = 21
    Warnid_Others = 22
    Warnid_Pure = 23
    Warnid_Analyze_Assert = 24
    Warnid_Static = 25
    Msgid_Warning = 26
    Msgid_Error = 27
    Msgid_Fatal = 28
