/* DO NOT MODIFY
   This file is automatically generated by Makefile.  */
enum Module_Id {
   Id_None = 0,
   Id_Free = 1,
   Id_Design = 2,
   Id_User_None  = 128,
   Id_User_First = Id_User_None + 1,
   Id_And  = 3,
   Id_Or   = 4,
   Id_Xor  = 5,
   Id_Nand = 6,
   Id_Nor  = 7,
   Id_Xnor = 8,
   Id_Add = 9,
   Id_Sub = 10,
   Id_Umul = 11,
   Id_Smul = 12,
   Id_Udiv = 13,
   Id_Sdiv = 14,
   Id_Umod = 15,
   Id_Smod = 16,
   Id_Srem = 17,
   Id_Not = 18,
   Id_Neg = 19,
   Id_Abs = 20,
   Id_Lsl = 22,
   Id_Lsr = 23,
   Id_Asr = 24,
   Id_Rol = 25,
   Id_Ror = 26,
   Id_Eq  = 27,
   Id_Ne  = 28,
   Id_Ule = 29,
   Id_Sle = 30,
   Id_Ult = 31,
   Id_Slt = 32,
   Id_Uge = 33,
   Id_Sge = 34,
   Id_Ugt = 35,
   Id_Sgt = 36,
   Id_Red_And = 37,
   Id_Red_Or  = 38,
   Id_Concat2 = 40,
   Id_Concat3 = 41,
   Id_Concat4 = 42,
   Id_Concatn = 43,
   Id_Mux2 = 44,
   Id_Mux4 = 45,
   Id_Signal  = 46,
   Id_Isignal = 47,
   Id_Output  = 48,
   Id_Port    = 49,
   Id_Dff   = 50,
   Id_Adff  = 51,
   Id_Idff  = 52,
   Id_Iadff = 53,
   Id_Mdff = 54,
   Id_Midff = 55,
   Id_Utrunc = 64,
   Id_Strunc = 65,
   Id_Uextend = 66,
   Id_Sextend = 67,
   Id_Extract = 68,
   Id_Dyn_Extract = 69,
   Id_Dyn_Insert = 70,
   Id_Dyn_Insert_En = 71,
   Id_Memidx = 72,
   Id_Addidx = 73,
   Id_Memory = 74,
   Id_Memory_Init = 75,
   Id_Mem_Rd = 76,
   Id_Mem_Rd_Sync = 77,
   Id_Mem_Wr_Sync = 78,
   Id_Edge = 80,
   Id_Assert = 81,
   Id_Assume = 82,
   Id_Cover = 83,
   Id_Assert_Cover = 84,
   Id_Allconst = 90,
   Id_Anyconst = 91,
   Id_Allseq = 92,
   Id_Anyseq = 93,
   Id_Const_UB32 = 112,
   Id_Const_SB32 = 113,
   Id_Const_UL32 = 114,
   Id_Const_UB64 = 115,
   Id_Const_UL64 = 116,
   Id_Const_X = 117,
   Id_Const_Z = 118,
   Id_Const_0 = 119,
   Id_Const_Bit = 120,
   Id_Const_Log = 121,
};
