from libghdl import libghdl

Strip_Denoting_Name = libghdl.vhdl__utils__strip_denoting_name

Get_Entity = libghdl.vhdl__utils__get_entity

Is_Second_Subprogram_Specification = \
    libghdl.vhdl__utils__is_second_subprogram_specification

Get_Entity_From_Entity_Aspect = \
    libghdl.vhdl__utils__get_entity_from_entity_aspect

Get_Interface_Of_Formal = \
    libghdl.vhdl__utils__get_interface_of_formal
